/**
 * @file        ADSO-9932
 * @description Manually create an operation feedback
 * @author      Kong Ching Shien (ckg6@3ds.com)
 * @copyright   Dassault Systèmes
 */

import { qCustomMatcher } from '../../e2elib/lib/src/main/qmatchers.const';
import { AppMP, Demo, Scenario } from '../../libmp/appmp';
import { ListEntity, ListEntityColumn, StepNavigationPanel } from '../../libmp/forms/navigationpanel/form.navigationpanel';
import { DialogOperationFeedback, DialogOperationFeedbackFields } from '../../libmp/dialogs/dialog.operationfeedback';
import { ListOperationFeedbackContextMenuItem, ListOperationFeedbackColumn, ListOperationFeedback } from '../../libmp/forms/form.operationfeedback';
import { StepDialog } from '../../libappsop/dialogsop';
import { startOfPlanningYear } from '../../libmp/data/data.period';
import { StepList } from '../../libappsop/listsop';
import { dataFoodUnits, dataFoodUnitsProvider } from '../../libmp/data/data.unit';

describe('ADSO-9932 - Manually create an operation feedback', () => {
   const appMP = AppMP.getInstance();

   const listOperationFeedback = appMP.viewFeedback.formOperationFeedback.listOperationFeedback;
   const deFermenter = dataFoodUnits.DEFermenter.Name;
   let dlgOperationFeedback: DialogOperationFeedback;
   const verifyOpFeedbackValue: DialogOperationFeedbackFields = { Unit: `${deFermenter}`, FeedbackLocked: 'true' };
   const updateOpFeedbackValue: DialogOperationFeedbackFields = { Date: `1-Apr-${startOfPlanningYear}`, Quantity: '1000', Description: 'Machine maintenance' };
   const verifyRow: ListOperationFeedbackColumn = { Unit: `${deFermenter}`, Routing: 'DE White Mass' , 'Routing Step': 'Ferment', Date: `1-Apr-${startOfPlanningYear}`, Quantity: '1,000', Description: 'Machine maintenance' };
   const verifyAvailableOpFeedbackValue: string[] = ['DE White Mass - Ferment - DE Fermenter'];
   const dlgNewOperationFeedback = new DialogOperationFeedback();
   const toggleEntityRow: ListEntityColumn = { DisplayName: deFermenter };

   beforeAll(async () => {
    jasmine.addMatchers(qCustomMatcher);
    await appMP.login();
   });

   afterAll(async () => {
     await appMP.resetActiveView(appMP.viewFeedback);
     await appMP.cleanupAndLogout();
   });

   afterEach(async () => {
     await appMP.checkToastMessage();
   });

   it(`Step 1 - ${AppMP.getDemoDataPath(Demo.Food, Scenario.Base)}.`, async () => {
     await appMP.createDemoDataset(Demo.Food, Scenario.Base, false);
   });

   it(`Step 2 - Open view ${appMP.viewFeedback.viewPath}.`, async () => {
     await appMP.viewFeedback.switchTo();
   });

   it(`Step 3 - ${StepNavigationPanel.showEntitiesList()}`, async () => {
    await appMP.formNavigation.toggleEntityList(true);
  });

   it(`Step 4 - ${StepList.toggleRowCheckbox(ListEntity.title, toggleEntityRow, true)}`, async () => {
    const row = await appMP.formNavigation.listEntity.getRow([dataFoodUnitsProvider, dataFoodUnits.DEFermenter]);
    await appMP.formNavigation.listEntity.toggleRowCheckbox(row, true);
  });

   it(`Step 5 - ${StepList.rightClickSelectMenu(ListOperationFeedback.title, ListOperationFeedbackContextMenuItem.Create.Label)} ${StepDialog.verifyDialogValues(DialogOperationFeedback.title, verifyOpFeedbackValue)}`, async () => {
    [dlgOperationFeedback] = await listOperationFeedback.selectContextMenu(ListOperationFeedbackContextMenuItem.Create);
    await dlgOperationFeedback.verifyDialogValues(verifyOpFeedbackValue);
  });

   it(`Step 6 - ${StepDialog.verifyAvailableValues(DialogOperationFeedback.title, dlgNewOperationFeedback.fieldName('Operation'), verifyAvailableOpFeedbackValue)}`, async () => {
    await dlgOperationFeedback.ddlOperation.verifyAvailableValues(verifyAvailableOpFeedbackValue);
  });

   it(`Step 7 - ${StepDialog.updateDialogValues(DialogOperationFeedback.title, updateOpFeedbackValue)} ${StepDialog.clickOK()}`, async () => {
    await dlgOperationFeedback.updateDialogValues(updateOpFeedbackValue);
    await dlgOperationFeedback.clickOK();
  });

   it(`Step 8 - ${StepList.verifyRowExists(ListOperationFeedback.title, verifyRow)}`, async () => {
    await listOperationFeedback.verifyRowExists(verifyRow);
  });

   it(`Step 9 - ${StepList.focusListClickActionButton(ListOperationFeedback.title, 'Create')} ${StepDialog.clickCancel()}`, async () => {
    // Purpose not following Jira test to re-create via action bar button (save execution time) and just ensure clicking Create button also shows the Create dialog.
    await listOperationFeedback.clickActionButton(appMP.abpPlan.btnCreate);
    await dlgOperationFeedback.clickCancel();
  });

 });
